import { useRef, useState, useEffect } from "react";
import type { PointerEvent } from "react";

type Props = {
  height: number;
  lines: number[][];
  onUpdateImage?: (path: number[]) => unknown;
  readonly?: boolean;
};

const cursorPos = (e: PointerEvent<HTMLCanvasElement>) => {
  const target = e.target as HTMLCanvasElement;
  const rect: DOMRect = target.getBoundingClientRect();
  return [e.clientX - rect.left, e.clientY - rect.top];
};

const DrawPad = ({ height, lines, onUpdateImage, readonly = false }: Props) => {
  const parent = useRef<HTMLElement>();
  const canvas = useRef<HTMLCanvasElement>();
  const [width, setWidth] = useState(0);
  const [down, setDown] = useState(false);
  const [path, setPath] = useState([]);
  const pen = canvas.current?.getContext("2d");

  const render = () => {
    if (pen) {
      pen.clearRect(0, 0, width, height);
      pen.fillStyle = "blue";
      pen.fillRect(0, 0, width, height);
      pen.lineWidth = 3;
      pen.lineCap = "round";
      pen.strokeStyle = "white";
      lines.forEach((path) => {
        if (path.length > 2) {
          pen.moveTo(path[0], path[1]);
        }
        for (let i = 2; i < path.length; i += 2) {
          pen.lineTo(path[i], path[i + 1]);
          pen.stroke();
        }
      });
    }
  };

  useEffect(() => {
    setWidth(parent.current?.getBoundingClientRect().width);
    render();
  }, [parent, pen, lines]);

  const beginLine = () => {
    if (readonly) return;
    pen.beginPath();
    setDown(true);
  };

  const drawLine = (e: PointerEvent<HTMLCanvasElement>) => {
    if (!down) return;
    const [x, y] = cursorPos(e);
    setPath([...path, x, y]);
    pen.lineTo(x, y);
    pen.stroke();
  };

  const endLine = () => {
    setDown(false);
    onUpdateImage?.(path);
    setPath([]);
  };

  return (
    <div ref={parent}>
      <canvas
        width={width}
        height={height}
        ref={canvas}
        onPointerDown={beginLine}
        onPointerMove={drawLine}
        onPointerUp={endLine}
        style={{ touchAction: "none" }}
      />
    </div>
  );
};

export default DrawPad;
